@@ -648,7 +648,7 @@ void xdebug_code_coverage_end_of_function(zend_op_array *op_array, zend_string *
 	xdebug_str str = XDEBUG_STR_INITIALIZER;
 	xdebug_path *path = xdebug_path_info_get_path_for_level(XG_COV(paths_stack), XG_BASE(level));
 
-	if (!path) {
+	if (!path || !path->elements) {
 		return;
 	}
 
@@ -1126,7 +1126,10 @@ void xdebug_coverage_minit(INIT_FUNC_ARGS)
 			xdebug_set_opcode_handler(i, xdebug_check_branch_entry_handler);
 		}
 	}
+}
 
+void xdebug_coverage_register_constants(INIT_FUNC_ARGS)
+{
 	REGISTER_LONG_CONSTANT("XDEBUG_CC_UNUSED", XDEBUG_CC_OPTION_UNUSED, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("XDEBUG_CC_DEAD_CODE", XDEBUG_CC_OPTION_DEAD_CODE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("XDEBUG_CC_BRANCH_CHECK", XDEBUG_CC_OPTION_BRANCH_CHECK, CONST_CS | CONST_PERSISTENT);
