@@ -22,7 +22,7 @@
 #include "zend_exceptions.h"
 
 #include "debugger_private.h"
-#include "base/stack.h"
+#include "lib/var.h"
 
 extern ZEND_DECLARE_MODULE_GLOBALS(xdebug);
 
@@ -31,23 +31,6 @@ static size_t xdebug_ub_write(const char *string, size_t length);
 
 static void xdebug_line_list_dtor(xdebug_lines_list *line_list);
 
-PHP_INI_MH(OnUpdateDebugMode)
-{
-	if (!new_value) {
-		XINI_DBG(remote_mode) = XDEBUG_NONE;
-
-	} else if (strcmp(STR_NAME_VAL(new_value), "jit") == 0) {
-		XINI_DBG(remote_mode) = XDEBUG_JIT;
-
-	} else if (strcmp(STR_NAME_VAL(new_value), "req") == 0) {
-		XINI_DBG(remote_mode) = XDEBUG_REQ;
-
-	} else {
-		XINI_DBG(remote_mode) = XDEBUG_NONE;
-	}
-	return SUCCESS;
-}
-
 void xdebug_init_debugger_globals(xdebug_debugger_globals_t *xg)
 {
 	xg->breakpoint_count     = 0;
@@ -55,7 +38,7 @@ void xdebug_init_debugger_globals(xdebug_debugger_globals_t *xg)
 	xg->stdout_mode          = 0;
 	xg->no_exec              = 0;
 	xg->context.program_name = NULL;
-	xg->context.list.last_file = NULL;
+	xg->context.list.last_filename = NULL;
 	xg->context.list.last_line = 0;
 	xg->context.do_break     = 0;
 	xg->context.do_step      = 0;
@@ -64,7 +47,6 @@ void xdebug_init_debugger_globals(xdebug_debugger_globals_t *xg)
 
 	xg->remote_connection_enabled = 0;
 	xg->remote_connection_pid     = 0;
-	xg->remote_log_file           = 0;
 	xg->breakpoints_allowed       = 0;
 
 	/* Capturing output */
@@ -88,16 +70,6 @@ static char *xdebug_debugger_get_ide_key(void)
 		return ide_key;
 	}
 
-	ide_key = getenv("USER");
-	if (ide_key && *ide_key) {
-		return ide_key;
-	}
-
-	ide_key = getenv("USERNAME");
-	if (ide_key && *ide_key) {
-		return ide_key;
-	}
-
 	return NULL;
 }
 
@@ -122,7 +94,7 @@ int xdebug_debugger_bailout_if_no_exec_requested(void)
 void xdebug_debugger_set_program_name(zend_string *filename)
 {
 	if (!XG_DBG(context).program_name) {
-		XG_DBG(context).program_name = xdstrdup(STR_NAME_VAL(filename));
+		XG_DBG(context).program_name = zend_string_copy(filename);
 	}
 }
 
@@ -165,8 +137,62 @@ static int xdebug_handle_hit_value(xdebug_brk_info *brk_info)
 	return 0;
 }
 
+int xdebug_do_eval(char *eval_string, zval *ret_zval)
+{
+#if PHP_VERSION_ID < 80000
+	int                old_track_errors;
+#endif
+	int                res = FAILURE;
+	zend_execute_data *original_execute_data = EG(current_execute_data);
+	int                original_no_extensions = EG(no_extensions);
+	zend_object       *original_exception = EG(exception);
+	JMP_BUF           *original_bailout = EG(bailout);
 
-void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
+	/* Remember error reporting level and track errors */
+	XG_BASE(error_reporting_override) = EG(error_reporting);
+	XG_BASE(error_reporting_overridden) = 1;
+#if PHP_VERSION_ID < 80000
+	old_track_errors = PG(track_errors);
+#endif
+	EG(error_reporting) = 0;
+#if PHP_VERSION_ID < 80000
+	PG(track_errors) = 0;
+#endif
+
+	XG_DBG(context).inhibit_notifications = 1;
+	XG_DBG(breakpoints_allowed) = 0;
+
+	/* Reset exception in case we're triggered while being in xdebug_throw_exception_hook */
+	EG(exception) = NULL;
+
+	/* Do evaluation */
+	zend_first_try {
+		res = zend_eval_string(eval_string, ret_zval, (char*) "xdebug://debug-eval");
+	} zend_end_try();
+
+	/* FIXME: Bubble up exception message to DBGp return packet */
+	if (EG(exception)) {
+		res = FAILURE;
+	}
+
+	/* Clean up */
+	EG(error_reporting) = XG_BASE(error_reporting_override);
+	XG_BASE(error_reporting_overridden) = 0;
+#if PHP_VERSION_ID < 80000
+	PG(track_errors) = old_track_errors;
+#endif
+	XG_DBG(breakpoints_allowed) = 1;
+	XG_DBG(context).inhibit_notifications = 0;
+
+	EG(current_execute_data) = original_execute_data;
+	EG(no_extensions) = original_no_extensions;
+	EG(exception) = original_exception;
+	EG(bailout) = original_bailout;
+
+	return res;
+}
+
+void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 {
 	xdebug_llist_element *le;
 	xdebug_brk_info      *extra_brk_info;
@@ -174,21 +200,19 @@ void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
 	int                   level = 0;
 	int                   func_nr = 0;
 
-	if (xdebug_is_debug_connection_active_for_current_pid()) {
-
+	if (xdebug_is_debug_connection_active()) {
 		if (XG_DBG(context).do_break) {
 			XG_DBG(context).do_break = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
 		}
 
 		/* Get latest stack level and function number */
-		if (XG_BASE(stack) && XDEBUG_LLIST_TAIL(XG_BASE(stack))) {
-			le = XDEBUG_LLIST_TAIL(XG_BASE(stack));
-			fse = XDEBUG_LLIST_VALP(le);
+		if (XG_BASE(stack) && XDEBUG_VECTOR_TAIL(XG_BASE(stack))) {
+			fse = XDEBUG_VECTOR_TAIL(XG_BASE(stack));
 			level = fse->level;
 			func_nr = fse->function_nr;
 		} else {
@@ -206,7 +230,7 @@ void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
 		) {
 			XG_DBG(context).do_finish = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -217,7 +241,7 @@ void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
 		if (XG_DBG(context).do_next && XG_DBG(context).next_level >= level) {
 			XG_DBG(context).do_next = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -228,7 +252,7 @@ void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
 		if (XG_DBG(context).do_step) {
 			XG_DBG(context).do_step = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -236,13 +260,13 @@ void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
 		}
 
 		if (XG_DBG(context).line_breakpoints) {
-			int   break_ok;
+			int   break_ok, res;
 			zval  retval;
 
 			for (le = XDEBUG_LLIST_HEAD(XG_DBG(context).line_breakpoints); le != NULL; le = XDEBUG_LLIST_NEXT(le)) {
 				extra_brk_info = XDEBUG_LLIST_VALP(le);
 
-				if (XG_DBG(context).handler->break_on_line(&(XG_DBG(context)), extra_brk_info, file, file_len, lineno)) {
+				if (XG_DBG(context).handler->break_on_line(&(XG_DBG(context)), extra_brk_info, filename, lineno)) {
 					break_ok = 1; /* Breaking is allowed by default */
 
 					/* Check if we have a condition set for it */
@@ -253,24 +277,14 @@ void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
 						break_ok = 0;
 
 						/* Remember error reporting level */
-						XG_BASE(error_reporting_override) = EG(error_reporting);
-						XG_BASE(error_reporting_overridden) = 1;
-						EG(error_reporting) = 0;
-						XG_DBG(context).inhibit_notifications = 1;
-
-						/* Check the condition */
-						if (zend_eval_string(extra_brk_info->condition, &retval, (char*) "xdebug conditional breakpoint") == SUCCESS) {
+						res = xdebug_do_eval(extra_brk_info->condition, &retval);
+						if (res == SUCCESS) {
 							break_ok = Z_TYPE(retval) == IS_TRUE;
 							zval_dtor(&retval);
 						}
-
-						/* Restore error reporting level */
-						EG(error_reporting) = XG_BASE(error_reporting_override);
-						XG_BASE(error_reporting_overridden) = 0;
-						XG_DBG(context).inhibit_notifications = 0;
 					}
 					if (break_ok && xdebug_handle_hit_value(extra_brk_info)) {
-						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
+						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
 							xdebug_mark_debug_connection_not_active();
 							break;
 						}
@@ -282,14 +296,21 @@ void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
 	}
 }
 
-void xdebug_debugger_throw_exception_hook(zend_class_entry * exception_ce, zval *file, zval *line, zval *code, char *code_str, zval *message)
+#if PHP_VERSION_ID >= 80000
+void xdebug_debugger_throw_exception_hook(zend_object *exception, zval *file, zval *line, zval *code, char *code_str, zval *message)
 {
+	zend_class_entry *exception_ce = exception->ce;
+#else
+void xdebug_debugger_throw_exception_hook(zval *exception, zval *file, zval *line, zval *code, char *code_str, zval *message)
+{
+	zend_class_entry *exception_ce = Z_OBJCE_P(exception);
+#endif
 	xdebug_brk_info *extra_brk_info;
 
 	/* Start JIT if requested and not yet enabled */
-	xdebug_do_jit();
+	xdebug_debug_init_if_requested_on_error();
 
-	if (xdebug_is_debug_connection_active_for_current_pid() && XG_DBG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active() && XG_DBG(breakpoints_allowed)) {
 		int exception_breakpoint_found = 0;
 
 		/* Check if we have a wild card exception breakpoint */
@@ -315,7 +336,7 @@ void xdebug_debugger_throw_exception_hook(zend_class_entry * exception_ce, zval
 		if (exception_breakpoint_found && xdebug_handle_hit_value(extra_brk_info)) {
 			if (!XG_DBG(context).handler->remote_breakpoint(
 				&(XG_DBG(context)), XG_BASE(stack),
-				Z_STRVAL_P(file), Z_LVAL_P(line), XDEBUG_BREAK,
+				Z_STR_P(file), Z_LVAL_P(line), XDEBUG_BREAK,
 				(char*) STR_NAME_VAL(exception_ce->name),
 				code_str ? code_str : ((code && Z_TYPE_P(code) == IS_STRING) ? Z_STRVAL_P(code) : NULL),
 				Z_STRVAL_P(message))
@@ -326,14 +347,14 @@ void xdebug_debugger_throw_exception_hook(zend_class_entry * exception_ce, zval
 	}
 }
 
-void xdebug_debugger_error_cb(const char *error_filename, int error_lineno, int type, char *error_type_str, char *buffer)
+void xdebug_debugger_error_cb(zend_string *error_filename, int error_lineno, int type, char *error_type_str, char *buffer)
 {
 	xdebug_brk_info *extra_brk_info = NULL;
 
 	/* Start JIT if requested and not yet enabled */
-	xdebug_do_jit();
+	xdebug_debug_init_if_requested_on_error();
 
-	if (xdebug_is_debug_connection_active_for_current_pid() && XG_DBG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active() && XG_DBG(breakpoints_allowed)) {
 		/* Send notification with warning/notice/error information */
 		if (XG_DBG(context).send_notifications && !XG_DBG(context).inhibit_notifications) {
 			if (!XG_DBG(context).handler->remote_notification(&(XG_DBG(context)), error_filename, error_lineno, type, error_type_str, buffer)) {
@@ -349,7 +370,7 @@ void xdebug_debugger_error_cb(const char *error_filename, int error_lineno, int
 			if (xdebug_handle_hit_value(extra_brk_info)) {
 				char *type_str = xdebug_sprintf("%ld", type);
 
-				if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), (char *) error_filename, error_lineno, XDEBUG_BREAK, error_type_str, type_str, buffer)) {
+				if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), error_filename, error_lineno, XDEBUG_BREAK, error_type_str, type_str, buffer)) {
 					xdebug_mark_debug_connection_not_active();
 				}
 
@@ -387,9 +408,9 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 	else if (fse->function.type == XFUNC_MEMBER || fse->function.type == XFUNC_STATIC_MEMBER) {
 		/* We intentionally do not use xdebug_sprintf because it can create a bottleneck in large
 		   codebases due to setlocale calls. We don't care about the locale here. */
-		tmp_len = strlen(fse->function.class) + strlen(fse->function.function) + 3;
+		tmp_len = ZSTR_LEN(fse->function.class_name) + strlen(fse->function.function) + 3;
 		tmp_name = xdmalloc(tmp_len);
-		snprintf(tmp_name, tmp_len, "%s::%s", fse->function.class, fse->function.function);
+		snprintf(tmp_name, tmp_len, "%s::%s", ZSTR_VAL(fse->function.class_name), fse->function.function);
 
 		if (xdebug_hash_find(XG_DBG(context).function_breakpoints, tmp_name, tmp_len - 1, (void *) &extra_brk_info)) {
 			/* Yup, breakpoint found, call handler if the breakpoint is not
@@ -413,7 +434,7 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 
 void xdebug_debugger_handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 {
-	if (xdebug_is_debug_connection_active_for_current_pid() && XG_DBG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active() && XG_DBG(breakpoints_allowed)) {
 		if (!handle_breakpoints(fse, breakpoint_type)) {
 			xdebug_mark_debug_connection_not_active();
 		}
@@ -422,7 +443,7 @@ void xdebug_debugger_handle_breakpoints(function_stack_entry *fse, int breakpoin
 
 static size_t xdebug_ub_write(const char *string, size_t length)
 {
-	if (xdebug_is_debug_connection_active_for_current_pid()) {
+	if (xdebug_is_debug_connection_active()) {
 		if (-1 == XG_DBG(context).handler->remote_stream_output(string, length)) {
 			return 0;
 		}
@@ -475,24 +496,7 @@ void xdebug_debugger_rinit(void)
 {
 	char *idekey;
 
-/* PHP Bug #77287 causes Xdebug to segfault if OPcache has the "compact
- * literals" optimisation turned on. So force the optimisation off for PHP
- * 7.3.0 and 7.3.1.
- *
- * Otherwise, only turn off optimisation when we're debugging. */
-#if PHP_VERSION_ID >= 70300 && PHP_VERSION_ID <= 70301
-	{
-#else
-	if (XINI_DBG(remote_enable)) {
-#endif
-		zend_string *key = zend_string_init(ZEND_STRL("opcache.optimization_level"), 1);
-		zend_string *value = zend_string_init(ZEND_STRL("0"), 1);
-
-		zend_alter_ini_entry(key, value, ZEND_INI_SYSTEM, ZEND_INI_STAGE_STARTUP);
-
-		zend_string_release(key);
-		zend_string_release(value);
-	}
+	xdebug_disable_opcache_optimizer();
 
 	/* Get the ide key for this session */
 	XG_DBG(ide_key) = NULL;
@@ -506,7 +510,6 @@ void xdebug_debugger_rinit(void)
 
 	XG_DBG(no_exec) = 0;
 	xdebug_lib_set_active_symbol_table(NULL);
-	xdebug_lib_set_active_object(NULL);
 
 	/* Check if we have this special get variable that stops a debugging
 	 * request without executing any code */
@@ -522,7 +525,7 @@ void xdebug_debugger_rinit(void)
 			)
 			&& !SG(headers_sent)
 		) {
-			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), (char*) "", 0, time(NULL) + XINI_DBG(remote_cookie_expire_time), "/", 1, NULL, 0, 0, 1, 0);
+			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), (char*) "", 0, time(NULL) + XDEBUG_COOKIE_EXPIRE_TIME, "/", 1, NULL, 0, 0, 1, 0);
 			XG_DBG(no_exec) = 1;
 		}
 		zend_string_release(stop_no_exec);
@@ -535,11 +538,10 @@ void xdebug_debugger_rinit(void)
 	XG_DBG(breakable_lines_map) = xdebug_hash_alloc(2048, (xdebug_hash_dtor_t) xdebug_line_list_dtor);
 	XG_DBG(function_count) = 0;
 	XG_DBG(class_count) = 0;
-	XG_DBG(remote_log_file) = NULL;
 
 	/* Initialize some debugger context properties */
 	XG_DBG(context).program_name   = NULL;
-	XG_DBG(context).list.last_file = NULL;
+	XG_DBG(context).list.last_filename = NULL;
 	XG_DBG(context).list.last_line = 0;
 	XG_DBG(context).do_break       = 0;
 	XG_DBG(context).do_step        = 0;
@@ -554,7 +556,7 @@ void xdebug_debugger_post_deactivate(void)
 		xdebug_close_socket(XG_DBG(context).socket);
 	}
 	if (XG_DBG(context).program_name) {
-		xdfree(XG_DBG(context).program_name);
+		zend_string_release(XG_DBG(context).program_name);
 	}
 
 	if (XG_DBG(ide_key)) {
@@ -562,12 +564,13 @@ void xdebug_debugger_post_deactivate(void)
 		XG_DBG(ide_key) = NULL;
 	}
 
-	if (XG_DBG(context.list.last_file)) {
-		xdfree(XG_DBG(context).list.last_file);
-		XG_DBG(context).list.last_file = NULL;
+	if (XG_DBG(context.list.last_filename)) {
+		zend_string_release(XG_DBG(context).list.last_filename);
+		XG_DBG(context).list.last_filename = NULL;
 	}
 
 	xdebug_hash_destroy(XG_DBG(breakable_lines_map));
+	XG_DBG(breakable_lines_map) = NULL;
 }
 
 xdebug_set *xdebug_debugger_get_breakable_lines_from_oparray(zend_op_array *opa)
@@ -672,7 +675,11 @@ void xdebug_debugger_compile_file(zend_op_array *op_array)
 	zend_class_entry *class_entry;
 	xdebug_lines_list *file_function_lines_list;
 
-	if (!XINI_DBG(remote_enable) || !XG_DBG(breakable_lines_map)) {
+	RETURN_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
+
+	/* The breakable_lines_map can not be set if another extension compiles
+	 * scripts during RINIT */
+	if (!XG_DBG(breakable_lines_map)) {
 		return;
 	}
 
@@ -702,7 +709,7 @@ void xdebug_debugger_compile_file(zend_op_array *op_array)
 
 	add_function_to_lines_list(file_function_lines_list, op_array);
 
-	if (!xdebug_is_debug_connection_active_for_current_pid()) {
+	if (!xdebug_is_debug_connection_active()) {
 		return;
 	}
 
@@ -723,7 +730,7 @@ static void resolve_breakpoints_for_eval(int eval_id, zend_op_array *opa)
 
 	resolve_breakpoints_for_function(lines_list, opa);
 
-	if (!xdebug_is_debug_connection_active_for_current_pid()) {
+	if (!xdebug_is_debug_connection_active()) {
 		zend_string_release(eval_string);
 		xdfree(eval_filename);
 		return;
@@ -740,19 +747,46 @@ static void resolve_breakpoints_for_eval(int eval_id, zend_op_array *opa)
 
 void xdebug_debugger_register_eval(function_stack_entry *fse)
 {
-	if (xdebug_is_debug_connection_active_for_current_pid() && XG_DBG(context).handler->register_eval_id) {
+	if (xdebug_is_debug_connection_active() && XG_DBG(context).handler->register_eval_id) {
 		int eval_id = XG_DBG(context).handler->register_eval_id(&(XG_DBG(context)), fse);
 
 		resolve_breakpoints_for_eval(eval_id, fse->op_array);
 	}
 }
 
+void xdebug_debugger_restart_if_pid_changed()
+{
+	zend_ulong pid;
+
+	if (!xdebug_is_debug_connection_active()) {
+		return;
+	}
+
+	pid = xdebug_get_pid();
+
+	/* Start debugger if previously a connection was established and this
+	 * process no longer has the same PID */
+	if (XG_DBG(remote_connection_pid) != pid) {
+		xdebug_restart_debugger();
+	}
+}
+
 
 PHP_FUNCTION(xdebug_break)
 {
-	/* Start JIT if requested and not yet enabled */
-	xdebug_do_jit();
+	RETURN_FALSE_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
+
+	xdebug_debug_init_if_requested_on_xdebug_break();
+
+	if (!xdebug_is_debug_connection_active()) {
+		RETURN_FALSE;
+	}
 
 	XG_DBG(context).do_break = 1;
 	RETURN_TRUE;
 }
+
+PHP_FUNCTION(xdebug_is_debugger_active)
+{
+	RETURN_BOOL(xdebug_is_debug_connection_active());
+}
