@@ -29,6 +29,7 @@
 #include "php_xdebug.h"
 
 #include "lib/compat.h"
+#include "lib/lib_private.h"
 #include "lib/mm.h"
 #include "lib/var.h"
 #include "lib/var_export_html.h"
@@ -178,12 +179,16 @@ char* xdebug_error_type(int type)
 
 zval *xdebug_get_zval_with_opline(zend_execute_data *zdata, const zend_op *opline, int node_type, const znode_op *node, int *is_var)
 {
+#if PHP_VERSION_ID >= 80000
+	return zend_get_zval_ptr(opline, node_type, node, zdata);
+#else
 	zend_free_op should_free;
 
-#if PHP_VERSION_ID >= 70300
+# if PHP_VERSION_ID >= 70300
 	return zend_get_zval_ptr(opline, node_type, node, zdata, &should_free, BP_VAR_IS);
-#else
+# else
 	return zend_get_zval_ptr(node_type, node, zdata, &should_free, BP_VAR_IS);
+# endif
 #endif
 }
 
@@ -474,7 +479,11 @@ static void fetch_zval_from_symbol_table(
 			/* First we try an object handler */
 			if (cce) {
 				zval *tmp_val;
+#if PHP_VERSION_ID >= 80000
+				tmp_val = zend_read_property(cce, Z_OBJ_P(value_in), name, name_length, 1, &tmp_retval);
+#else
 				tmp_val = zend_read_property(cce, value_in, name, name_length, 1, &tmp_retval);
+#endif
 				if (tmp_val != &tmp_retval && tmp_val != &EG(uninitialized_zval)) {
 					ZVAL_COPY(&tmp_retval, tmp_val);
 					goto cleanup;
@@ -827,8 +836,8 @@ void xdebug_get_php_symbol(zval *retval, xdebug_str* name)
 						state = 1;
 						keyword_end = &ptr[ctr];
 
-						if (strncmp(keyword, "::", 2) == 0 && active_fse->function.class) { /* static class properties */
-							zend_class_entry *ce = xdebug_fetch_class(active_fse->function.class, strlen(active_fse->function.class), ZEND_FETCH_CLASS_SELF);
+						if (strncmp(keyword, "::", 2) == 0 && active_fse->function.class_name) { /* static class properties */
+							zend_class_entry *ce = zend_fetch_class(active_fse->function.class_name, ZEND_FETCH_CLASS_SELF);
 
 							current_classname = estrdup(STR_NAME_VAL(ce->name));
 							cc_length = strlen(STR_NAME_VAL(ce->name));
@@ -890,100 +899,6 @@ void xdebug_get_php_symbol(zval *retval, xdebug_str* name)
 	}
 }
 
-#define XDEBUG_VAR_FORMAT_INITIALISED   0
-#define XDEBUG_VAR_FORMAT_UNINITIALISED 1
-
-static const char* text_formats[2] = {
-	"  $%s = %s\n",
-	"  $%s = *uninitialized*\n",
-};
-
-static const char* ansi_formats[2] = {
-	"  $%s = %s\n",
-	"  $%s = *uninitialized*\n",
-};
-
-static const char* html_formats[2] = {
-	"<tr><td colspan='2' align='right' bgcolor='#eeeeec' valign='top'><pre>$%s&nbsp;=</pre></td><td colspan='3' bgcolor='#eeeeec'>%s</td></tr>\n",
-	"<tr><td colspan='2' align='right' bgcolor='#eeeeec' valign='top'><pre>$%s&nbsp;=</pre></td><td colspan='3' bgcolor='#eeeeec' valign='top'><i>Undefined</i></td></tr>\n",
-};
-
-
-static const char** get_var_format_string(int html)
-{
-	if (html) {
-		return html_formats;
-	}
-	else if ((XINI_BASE(cli_color) == 1 && xdebug_is_output_tty()) || (XINI_BASE(cli_color) == 2)) {
-		return ansi_formats;
-	}
-	else {
-		return text_formats;
-	}
-}
-
-void xdebug_dump_used_var_with_contents(void *htmlq, xdebug_hash_element* he, void *argument)
-{
-	int          html = *(int*) htmlq;
-	zval         zvar;
-	xdebug_str  *contents;
-	xdebug_str  *name = (xdebug_str*) he->ptr;
-	HashTable   *tmp_ht;
-	const char **formats;
-	xdebug_str   *str = (xdebug_str *) argument;
-
-	if (!he->ptr) {
-		return;
-	}
-
-	/* Bail out on $this and $GLOBALS */
-	if (strcmp(name->d, "this") == 0 || strcmp(name->d, "GLOBALS") == 0) {
-		return;
-	}
-
-	if (!(ZEND_CALL_INFO(EG(current_execute_data)) & ZEND_CALL_HAS_SYMBOL_TABLE)) {
-		zend_rebuild_symbol_table();
-	}
-
-	tmp_ht = xdebug_lib_get_active_symbol_table();
-	{
-		zend_execute_data *ex = EG(current_execute_data);
-		while (ex && (!ex->func || !ZEND_USER_CODE(ex->func->type))) {
-			ex = ex->prev_execute_data;
-		}
-		if (ex) {
-			xdebug_lib_set_active_data(ex);
-			xdebug_lib_set_active_symbol_table(ex->symbol_table);
-		}
-	}
-
-	xdebug_get_php_symbol(&zvar, name);
-	xdebug_lib_set_active_symbol_table(tmp_ht);
-
-	formats = get_var_format_string(PG(html_errors));
-
-	if (Z_TYPE(zvar) == IS_UNDEF) {
-		xdebug_str_add(str, xdebug_sprintf(formats[XDEBUG_VAR_FORMAT_UNINITIALISED], name->d), 1);
-		return;
-	}
-
-	if (html) {
-		contents = xdebug_get_zval_value_html(NULL, &zvar, 0, NULL);
-	} else {
-		contents = xdebug_get_zval_value_line(&zvar, 0, NULL);
-	}
-
-	if (contents) {
-		xdebug_str_add(str, xdebug_sprintf(formats[XDEBUG_VAR_FORMAT_INITIALISED], name->d, contents->d), 1);
-	} else {
-		xdebug_str_add(str, xdebug_sprintf(formats[XDEBUG_VAR_FORMAT_UNINITIALISED], name->d), 1);
-	}
-
-	if (contents) {
-		xdebug_str_free(contents);
-	}
-	zval_ptr_dtor_nogc(&zvar);
-}
 
 #if PHP_VERSION_ID >= 70400
 xdebug_str* xdebug_get_property_type(zval* object, zval *val)
@@ -999,6 +914,11 @@ xdebug_str* xdebug_get_property_type(zval* object, zval *val)
 	info = zend_get_typed_property_info_for_slot(Z_OBJ_P(object), val);
 
 	if (info) {
+#if PHP_VERSION_ID >= 80000
+		zend_string *type_info = zend_type_to_string(info->type);
+		type_str = xdebug_str_create(ZSTR_VAL(type_info), ZSTR_LEN(type_info));
+		zend_string_release(type_info);
+#else
 		type_str = xdebug_str_new();
 
 		if (ZEND_TYPE_ALLOW_NULL(info->type)) {
@@ -1015,6 +935,7 @@ xdebug_str* xdebug_get_property_type(zval* object, zval *val)
 		} else {
 			xdebug_str_add(type_str, zend_get_type_by_const(ZEND_TYPE_CODE(info->type)), 0);
 		}
+#endif
 	}
 
 	return type_str;
@@ -1053,11 +974,11 @@ xdebug_var_export_options* xdebug_var_export_options_from_ini(void)
 	xdebug_var_export_options *options;
 	options = xdmalloc(sizeof(xdebug_var_export_options));
 
-	options->max_children = XINI_BASE(display_max_children);
-	options->max_data = XINI_BASE(display_max_data);
-	options->max_depth = XINI_BASE(display_max_depth);
+	options->max_children = XINI_LIB(display_max_children);
+	options->max_data = XINI_LIB(display_max_data);
+	options->max_depth = XINI_LIB(display_max_depth);
 	options->show_hidden = 0;
-	options->show_location = XINI_BASE(overload_var_dump) > 1;
+	options->show_location = 1;
 	options->extended_properties = 0;
 	options->encode_as_extended_property = 0;
 
@@ -1102,28 +1023,28 @@ xdebug_var_export_options* xdebug_var_get_nolimit_options(void)
 void xdebug_add_variable_attributes(xdebug_str *str, zval *struc, zend_bool html)
 {
 	if (html) {
-		xdebug_str_add(str, "<i>(", 0);
+		xdebug_str_add_literal(str, "<i>(");
 	} else {
-		xdebug_str_add(str, "(", 0);
+		xdebug_str_add_literal(str, "(");
 	}
 
 	if (Z_TYPE_P(struc) >= IS_STRING && Z_TYPE_P(struc) != IS_INDIRECT) {
 		if (Z_TYPE_P(struc) == IS_STRING && ZSTR_IS_INTERNED(Z_STR_P(struc))) {
-			xdebug_str_add(str, "interned", 0);
+			xdebug_str_add_literal(str, "interned");
 		} else if (Z_TYPE_P(struc) == IS_ARRAY && (GC_FLAGS(Z_ARRVAL_P(struc)) & IS_ARRAY_IMMUTABLE)) {
-			xdebug_str_add(str, "immutable", 0);
+			xdebug_str_add_literal(str, "immutable");
 		} else {
-			xdebug_str_add(str, xdebug_sprintf("refcount=%d", Z_REFCOUNT_P(struc)), 1);
+			xdebug_str_add_fmt(str, "refcount=%d", Z_REFCOUNT_P(struc));
 		}
-		xdebug_str_add(str, xdebug_sprintf(", is_ref=%d", Z_TYPE_P(struc) == IS_REFERENCE), 1);
+		xdebug_str_add_fmt(str, ", is_ref=%d", Z_TYPE_P(struc) == IS_REFERENCE);
 	} else {
-		xdebug_str_add(str, "refcount=0, is_ref=0", 0);
+		xdebug_str_add_literal(str, "refcount=0, is_ref=0");
 	}
 
 	if (html) {
-		xdebug_str_add(str, ")</i>", 0);
+		xdebug_str_add_literal(str, ")</i>");
 	} else {
-		xdebug_str_add(str, ")=", 0);
+		xdebug_str_add_literal(str, ")=");
 	}
 }
 
@@ -1131,40 +1052,78 @@ void xdebug_add_variable_attributes(xdebug_str *str, zval *struc, zend_bool html
 /*****************************************************************************
 ** XML encoding function
 */
+static const char xml_encode_count[256] = {
+	4, 1, 1, 1,  1, 1, 1, 1,  1, 1, 5, 1,  1, 5, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 6, 1,  1, 1, 5, 5,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  4, 1, 4, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+	1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
+};
 
-char* xdebug_xmlize(char *string, size_t len, size_t *newlen)
+static const char* xml_encode_map[64] = {
+	"&#0;", NULL, NULL,     NULL,  NULL, NULL, NULL,    NULL,     NULL, NULL, "&#10;", NULL,  NULL,   "&#13;", NULL,   NULL,
+	NULL,   NULL, NULL,     NULL,  NULL, NULL, NULL,    NULL,     NULL, NULL, NULL,    NULL,  NULL,   NULL,    NULL,   NULL,
+	NULL,   NULL, "&quot;", NULL,  NULL, NULL, "&amp;", "&#39;",  NULL, NULL, NULL,    NULL,  NULL,   NULL,    NULL,   NULL,
+	NULL,   NULL, NULL,     NULL,  NULL, NULL, NULL,    NULL,     NULL, NULL, NULL,    NULL,  "&lt;", NULL,    "&gt;", NULL,
+};
+
+char* xdebug_xmlize(char *s_string, size_t len, size_t *newlen)
 {
-	if (len) {
-		char *tmp;
-		char *tmp2;
+	int i, w_pos;
+	int encoded_string_length = 0;
+	char *new_string;
+	const unsigned char *string = (unsigned char*) s_string;
 
-		tmp = xdebug_str_to_str(string, len, "&", 1, "&amp;", 5, &len);
-
-		tmp2 = xdebug_str_to_str(tmp, len, ">", 1, "&gt;", 4, &len);
-		efree(tmp);
-
-		tmp = xdebug_str_to_str(tmp2, len, "<", 1, "&lt;", 4, &len);
-		efree(tmp2);
-
-		tmp2 = xdebug_str_to_str(tmp, len, "\"", 1, "&quot;", 6, &len);
-		efree(tmp);
-
-		tmp = xdebug_str_to_str(tmp2, len, "'", 1, "&#39;", 5, &len);
-		efree(tmp2);
-
-		tmp2 = xdebug_str_to_str(tmp, len, "\n", 1, "&#10;", 5, &len);
-		efree(tmp);
-
-		tmp = xdebug_str_to_str(tmp2, len, "\r", 1, "&#13;", 5, &len);
-		efree(tmp2);
-
-		tmp2 = xdebug_str_to_str(tmp, len, "\0", 1, "&#0;", 4, (size_t*) newlen);
-		efree(tmp);
-		return tmp2;
-	} else {
-		*newlen = len;
-		return estrdup(string);
+	/* Quick bailout for empty strings */
+	if (!len) {
+		*newlen = 0;
+		return estrdup("");
 	}
+
+	/* Calculate new memory requirement */
+	for (i = 0; i < len; i++) {
+		encoded_string_length += xml_encode_count[string[i]];
+	}
+
+	/* No characters need to be encoded, so just duplicate and return */
+	if (encoded_string_length == len) {
+		*newlen = len;
+		return estrdup(s_string);
+	}
+
+	new_string = emalloc(encoded_string_length + 1);
+	w_pos = 0;
+	for (i = 0; i < len; i++) {
+		int replacement_length = xml_encode_count[string[i]];
+
+		if (replacement_length != 1) {
+			int j;
+
+			for (j = 0; j < replacement_length; j++) {
+				new_string[w_pos] = xml_encode_map[string[i]][j];
+				w_pos++;
+			}
+			continue;
+		}
+
+		new_string[w_pos] = string[i];
+		w_pos++;
+	}
+
+	new_string[w_pos] = '\0';
+	*newlen = encoded_string_length; /* remove one for null byte */
+
+	return new_string;
 }
 
 /*****************************************************************************
@@ -1183,9 +1142,9 @@ static char* xdebug_create_doc_link(xdebug_func f)
 		case XFUNC_STATIC_MEMBER:
 		case XFUNC_MEMBER: {
 			if (strcmp(f.function, "__construct") == 0) {
-				tmp_target = xdebug_sprintf("%s.construct", f.class);
+				tmp_target = xdebug_sprintf("%s.construct", ZSTR_VAL(f.class_name));
 			} else {
-				tmp_target = xdebug_sprintf("%s.%s", f.class, f.function);
+				tmp_target = xdebug_sprintf("%s.%s", ZSTR_VAL(f.class_name), f.function);
 			}
 			break;
 		}
@@ -1222,7 +1181,7 @@ char* xdebug_show_fname(xdebug_func f, int html, int flags)
 				return xdebug_create_doc_link(f);
 			} else {
 				return xdebug_sprintf("%s%s%s",
-					f.class ? f.class : "?",
+					f.class_name ? ZSTR_VAL(f.class_name) : "?",
 					f.type == XFUNC_STATIC_MEMBER ? "::" : "->",
 					f.function ? f.function : "?"
 				);
